<?php defined( 'ABSPATH' ) OR die( 'This script cannot be accessed directly.' );
/**
 * Description of schema-editor
 *
 * @author Mark van Berkel
 */
class SchemaFront
{
    public $Settings;
    public $wpseo_meta_description = '';

    /**
     * Hook into the appropriate actions when the class is constructed.
     */
    public function __construct()
    {
		$this->Settings = get_option( 'schema_option_name' );

		add_action( 'plugins_loaded', array( $this, 'hook_plugins_loaded' ) );
		add_action( 'init', array( $this, 'HandleCache' ) );
		add_action( 'rest_api_init', array( $this, 'hook_rest_api_init' ) );

		if ( ! empty( $this->Settings['SchemaLinkedOpenData'] ) ) {
			add_action( 'send_headers', array( $this, 'linked_open_data_headers' ) );
			add_action( 'wp', array( $this, 'linked_open_data_output' ), 10, 1 );
			add_action( 'wp_head', array( $this, 'linked_open_data_link_tag' ) );
		}

		// Do not change priority of following hooks as it breaks hook chaining and functions like wp_localize_script
		if ( ! empty( $this->Settings['SchemaDefaultLocation'] ) && $this->Settings['SchemaDefaultLocation'] == 'Footer' )
		{
			add_action( 'wp_footer', array( $this, 'hunch_schema_add' ) );
		}
		else
		{
			add_action( 'wp_head', array( $this, 'hunch_schema_add' ) );
		}

		if ( ! empty( $this->Settings['SchemaRemoveMicrodata'] ) )
		{
			add_action( 'template_redirect', array( $this, 'TemplateRedirect' ), 0 );
		}

		if ( ! empty( $this->Settings['ToolbarShowTestSchema'] ) )
		{
			add_action( 'admin_bar_menu', array( $this, 'AdminBarMenu' ), 999 );
		}

		// Priority 15 ensures it runs after Genesis itself has setup.
		add_action( 'genesis_setup', array( $this, 'GenesisSetup' ), 15 );

		add_action( 'amp_post_template_head', array( $this, 'AMPPostTemplateHead' ) );
		add_filter( 'amp_post_template_metadata', '__return_false', 100 );
		add_filter( 'amp_schemaorg_metadata', '__return_false', 100 );
    }


	public function hook_plugins_loaded() {
		if ( defined( 'WPSEO_VERSION' ) ) {
			// Default enabled
			if ( ! isset( $this->Settings['SchemaRemoveWPSEOMarkup'] ) || $this->Settings['SchemaRemoveWPSEOMarkup'] == 1 ) {
				//https://developer.yoast.com/features/schema/api/
				add_filter( 'wpseo_json_ld_output', '__return_false', 100 );
			}

			if ( WPSEO_VERSION >= 14 ) {
				add_filter( 'wpseo_metadesc', array( $this, 'wpseo_meta_description' ), 10, 2 );
			}
		}
	}


	public function HandleCache()
	{
		if ( isset( $_GET['Action'], $_GET['URL'] ) && $_GET['Action'] == 'HSDeleteMarkupCache' )
		{
			delete_transient( 'HunchSchema-Markup-' . md5( $_GET['URL'] ) );

			header( 'HTTP/1.0 202 Accepted', true, 202 );

			exit;
		}
	}


	public function hook_rest_api_init() {
		register_rest_route( 'hunch_schema', '/cache/', array(
			array( 'methods' => 'GET', 'callback' => array( $this, 'rest_api_cache' ), 'permission_callback' => '__return_true' ),
			array( 'methods' => 'POST', 'callback' => array( $this, 'rest_api_cache_modify' ), 'permission_callback' => '__return_true' ),
		) );
	}


	public function rest_api_cache( $request ) {
		return new WP_REST_Response( array( 'status' => 'error', 'message' => 'Please use POST method to send data' ), 405 );
	}


	public function rest_api_cache_modify( $request ) {
		// Log request when debug is on
		if ( ! empty( $this->Settings['Debug'] ) ) {
			$upload_dir_param			= wp_upload_dir( null, false );
			$rest_api_cache_log_file	= $upload_dir_param['basedir'] . DIRECTORY_SEPARATOR . 'schema_app_rest_api_cache_log.txt';

			$this->create_log( $rest_api_cache_log_file, sprintf( "Body:\n%s\n\n\n", $request->get_body() ) );
		}

		$request_data	= json_decode( $request->get_body() );
		$site_domain	= str_replace( array( 'http://', 'https://' ), '', site_url() );
		$home_domain	= str_replace( array( 'http://', 'https://' ), '', home_url() );

		if ( empty( $request->get_body() ) || empty( $request_data ) ) {
			if ( ! empty( $this->Settings['Debug'] ) ) {
				$this->create_log( $rest_api_cache_log_file, 'Result: Invalid JSON data' );
			}

			return new WP_REST_Response( array( 'status' => 'error', 'message' => 'Invalid JSON data' ), 400 );
		}

		// Check for Account Id
		$account_id			= trim( str_replace( array( 'http://schemaapp.com/db/', 'https://schemaapp.com/db/' ), '', $this->Settings['graph_uri'] ), '/' );
		$request_account_id	= trim( str_ireplace( array( 'https://data.schemaapp.com/', $request_data->{"base64encode"} ), '', $request_data->{"url"} ), '/' );

		if ( $request_account_id != $account_id ) {
			if ( ! empty( $this->Settings['Debug'] ) ) {
				$this->create_log( $rest_api_cache_log_file, sprintf( 'Result: Invalid url property, Account Id does not match. url: %s Account Id: %s', $request_data->{"url"}, $account_id ) );
			}

			return new WP_REST_Response( array( 'status' => 'error', 'message' => 'Invalid url property, Account Id does not match' ), 401 );
		}

		// @graph is empty for EntityDeleted
		if ( empty( $request_data->{"@type"} ) || empty( $request_data->{"@id"} ) || ( $request_data->{"@type"} != 'EntityDeleted' && empty( $request_data->{"@graph"} ) ) ) {
			if ( ! empty( $this->Settings['Debug'] ) ) {
				$this->create_log( $rest_api_cache_log_file, 'Result: Invalid @type, @id or @graph property' );
			}

			return new WP_REST_Response( array( 'status' => 'error', 'message' => 'Invalid @type, @id or @graph property' ), 400 );
		}

		if ( stripos( $request_data->{"@id"}, $site_domain ) === false && stripos( $request_data->{"@id"}, $home_domain ) === false ) {
			if ( ! empty( $this->Settings['Debug'] ) ) {
				$this->create_log( $rest_api_cache_log_file, sprintf( 'Result: Invalid @id property, url does not match. @id: %s Site Domain: %s Home Domain: %s', $request_data->{"@id"}, $site_domain, $home_domain ) );
			}

			return new WP_REST_Response( array( 'status' => 'error', 'message' => 'Invalid @id property, url does not match' ), 400 );
		}


		// Remove anchor from permalink so that cache hash matches WP url
		if ( stripos( $request_data->{"@id"}, '#' ) !== false ) {
			$permalink = strstr( $request_data->{"@id"}, '#', true );
		} else {
			$permalink = $request_data->{"@id"};
		}

		$transient_id = 'HunchSchema-Markup-' . md5( $permalink );

		if ( ! empty( $this->Settings['Debug'] ) ) {
			$this->create_log( $rest_api_cache_log_file, sprintf( "Permalink: %s\nTransient: %s", $permalink, $transient_id ) );
		}

		switch ( $request_data->{"@type"} ) {
			case 'EntityCreated':
			case 'EntityUpdated':
				// First delete then set; set method only updates expiry time if transient already exists
				delete_transient( $transient_id );
				set_transient( $transient_id, json_encode( $request_data->{"@graph"} ), 86400 );

				if ( ! empty( $this->Settings['Debug'] ) ) {
					$this->create_log( $rest_api_cache_log_file, 'Result: Cache updated' );
				}

				return new WP_REST_Response( array( 'status' => 'ok', 'message' => 'Cache updated' ), 200 );
				break;
			case 'EntityDeleted':
				delete_transient( $transient_id );

				if ( ! empty( $this->Settings['Debug'] ) ) {
					$this->create_log( $rest_api_cache_log_file, 'Result: Cache deleted' );
				}

				return new WP_REST_Response( array( 'status' => 'ok', 'message' => 'Cache deleted' ), 200 );
				break;
			default:
				if ( ! empty( $this->Settings['Debug'] ) ) {
					$this->create_log( $rest_api_cache_log_file, 'Result: Invalid @type property' );
				}

				return new WP_REST_Response( array( 'status' => 'error', 'message' => 'Invalid @type property' ), 400 );
		}
	}


    /**
     * hunch_schema_add is called to lookup schema.org or add default markup 
     */
    public function hunch_schema_add( $JSON = false )
    {
		global $post;

		if ( empty( $post ) )
		{
			return;
		}

		$PostType = get_post_type();


		if ( is_singular() ) {
			$global_markup			= true;
			$single_markup_disable	= get_post_meta( $post->ID, '_HunchSchemaDisableMarkup', true );
			$single_markup_enable	= get_post_meta( $post->ID, '_HunchSchemaEnableMarkup', true );

			if ( $PostType == 'page' && isset( $this->Settings['SchemaDefaultShowOnPage'] ) && $this->Settings['SchemaDefaultShowOnPage'] == 0 ) {
				$global_markup = false;
			}

			if ( $PostType == 'post' && isset( $this->Settings['SchemaDefaultShowOnPost'] ) && $this->Settings['SchemaDefaultShowOnPost'] == 0 ) {
				$global_markup = false;
			}

			if (  ( $global_markup && $single_markup_disable )  ||  ( ! $global_markup && ! $single_markup_enable )  ) {
				return;
			}
		}


		$SchemaThing = HunchSchema_Thing::factory( $PostType );
		$SchemaServer = new SchemaServer();
		$SchemaMarkup = $SchemaServer->getResource();

		$JSONSchemaMarkup = array();
		$SchemaMarkupType = '';

		if ( empty( $SchemaMarkup ) ) {
			$SchemaMarkupCustom = get_post_meta( $post->ID, '_HunchSchemaMarkup', true );

			if ( $SchemaMarkupCustom )
			{
				$SchemaMarkupType = 'Custom';
				$SchemaMarkup = $SchemaMarkupCustom;
			}
			else if ( isset( $SchemaThing ) )
			{
				$SchemaMarkupType = 'Default';
				$SchemaMarkup = $SchemaThing->getResource();
			}
		}
		else
		{
			$SchemaMarkupType = 'App';
		}

		do_action( 'hunch_schema_markup_render', $SchemaMarkup, $SchemaMarkupType, $post, $PostType, $JSON );

		$SchemaMarkup = apply_filters( 'hunch_schema_markup', $SchemaMarkup, $SchemaMarkupType, $post, $PostType );

		if ( $SchemaMarkup !== "" && ! is_null( $SchemaMarkup ) )
		{
			if ( $JSON )
			{
				$JSONSchemaMarkup[] = json_decode( $SchemaMarkup );
			}
			else
			{
				if (  ( ! empty( $this->Settings['Debug'] ) && is_user_logged_in() )  ||  isset( $_GET['hunch_schema_debug'] )  ) {
					printf( "\n<!--\nURL: %s\nTransient: %s\nTransient Id: %s\nData Sources:\n", $SchemaServer->resource_url, ( $SchemaServer->transient ? 'Yes' : 'No' ), $SchemaServer->transient_id );

					foreach ( $SchemaServer->data_sources as $data_source ) {
						printf( "%s\n", $data_source );
					}

					print "-->\n";
				}

				printf( '<script type="application/ld+json" data-schema="%s-%s-%s">%s</script>' . "\n", $post->ID, $PostType, $SchemaMarkupType, $SchemaMarkup );
			}
		}

		if ( ! empty( $this->Settings['SchemaWebSite'] ) && is_front_page() )
		{
			$SchemaMarkupWebSite = apply_filters( 'hunch_schema_markup_website', $SchemaThing->getWebSite(), $PostType );

			if ( ! empty( $SchemaMarkupWebSite ) )
			{
				if ( $JSON )
				{
					$JSONSchemaMarkup[] = json_decode( $SchemaMarkupWebSite );
				}
				else
				{
					printf( '<script type="application/ld+json" data-schema="Website">%s</script>' . "\n", $SchemaMarkupWebSite );
				}
			}
		}

		if ( ! empty( $this->Settings['SchemaBreadcrumb'] ) && method_exists( $SchemaThing, 'getBreadcrumb' ) )
		{
			$SchemaMarkupBreadcrumb = apply_filters( 'hunch_schema_markup_breadcrumb', $SchemaThing->getBreadcrumb(), $PostType );

			if ( ! empty( $SchemaMarkupBreadcrumb ) )
			{
				if ( $JSON )
				{
					$JSONSchemaMarkup[] = json_decode( $SchemaMarkupBreadcrumb );
				}
				else
				{
					printf( '<script type="application/ld+json" data-schema="Breadcrumb">%s</script>' . "\n", $SchemaMarkupBreadcrumb );
				}
			}
		}

		if ( $JSON && ! empty( $JSONSchemaMarkup ) )
		{
			if ( count( $JSONSchemaMarkup ) == 1 )
			{
				$JSONSchemaMarkup = reset( $JSONSchemaMarkup );

				print json_encode( $JSONSchemaMarkup );
			}
			else
			{
				print json_encode( $JSONSchemaMarkup );
			}
		}
    }


	public function linked_open_data_output( $wp ) {
		$request_headers = array();

		if ( function_exists( 'apache_request_headers' ) ) {	
			$request_headers = apache_request_headers();
		}

		if (  ( ! empty( $_GET['format'] ) && $_GET['format'] == 'application/ld json' )  ||  ( ! empty( $request_headers['Accept'] ) && $request_headers['Accept'] == 'application/ld+json' )  ) {
			$this->hunch_schema_add( true );

			exit;
		}
	}


	public function linked_open_data_headers( $wp ) {
		if ( ! empty( $_GET['format'] ) && $_GET['format'] == 'application/ld json' ) {
			header( 'Content-Type: application/json; charset=UTF-8', true );
		}
	}


	public function linked_open_data_link_tag() {
		printf( '<link rel="alternate" type="application/ld+json" href="%s?format=application/ld+json" title="Structured Descriptor Document (JSON-LD format)">', esc_url( HunchSchema_Thing::getPermalink() ) );
	}


	public function TemplateRedirect()
	{
		ob_start( array( $this, 'RemoveMicrodata' ) );
	}


	public function RemoveMicrodata( $Buffer )
	{
		$Buffer = preg_replace( '/[\s\n]*<(link|meta)(\s|[^>]+\s)itemprop=[\'"][^\'"]*[\'"][^>]*>[\s\n]*/imS', '', $Buffer );

		for ( $I = 1; $I <= 6; $I++ )
		{
			$Buffer = preg_replace( '/(<[^>]*)\sitem(scope|type|prop)(=[\'"][^\'"]*[\'"])?([^>]*>)/imS', '$1$4', $Buffer );
		}

		return $Buffer;
	}


	public function AdminBarMenu( $WPAdminBar )
	{
		$Permalink = HunchSchema_Thing::getPermalink();

		if ( $Permalink )
		{
			$Node = array
			(
				'id'    => 'Hunch-Schema',
				'title' => 'Test Schema',
				'href'  => 'https://search.google.com/structured-data/testing-tool#url=' . urlencode( esc_url( $Permalink ) ),
				'meta'  => array
				(
					'class' => 'Hunch-Schema',
					'target' => '_blank',
				),
			);

			$WPAdminBar->add_node( $Node );
		}
	}


	public function GenesisSetup()
	{
		$Attributes = get_option( 'schema_option_name_genesis' );

		if ( $Attributes )
		{
			foreach ( $Attributes as $Key => $Value )
			{
				add_filter( 'genesis_attr_' . $Key, array( $this, 'GenesisAttribute' ), 20 );
			}
		}
	}


	public function GenesisAttribute( $Attribute )
	{
		$Attribute['itemtype'] = '';
		$Attribute['itemprop'] = '';
		$Attribute['itemscope'] = '';

		return $Attribute;
	}


	public function AMPPostTemplateHead( $Template )
	{
		$this->hunch_schema_add( false );
	}


	public function wpseo_meta_description( $meta_description, $presentation ) {
		$this->wpseo_meta_description = $meta_description;

		return $meta_description;
	}


	/**
	 * Function to create a log file. By default it limits file size to 1 Mb and prepend the passed data to file. If file does not exist then creates the file. Adds full date/time to log message.
	 *
	 * Uses file_get_contents and file_put_contents functions.
	 *
	 * @param string $file		Full path to log file.
	 * @param string $message	Log message.
	 * @param boolean $append	This parameter decides whether log message will be appende or prepend to log file. If set to true then $message is appended to log file and $length parameter is ignored. 
	 *							If set to false then $message is prepended to log file.
	 * @param integer $length	This parameter limits the log file size in byte, default to 1 Mega byte.
	 * @return file_put_contents return value: integer number of bytes that were written to the file, or boolean FALSE on failure.
	 */
	public function create_log( $file, $message = '', $append = false, $length = 1048576 ) {
		$message = sprintf( "Time: %s\n%s", date( 'c' ), $message );

		if ( file_exists( $file ) ) {
			if ( $append ) {
				return file_put_contents( $file, $message, FILE_APPEND );
			} else {
				if ( $length ) {
					$content = file_get_contents( $file, false, null, 0, $length );
				} else {
					$content = file_get_contents( $file );
				}

				return file_put_contents( $file, "$message\n\n$content" );
			}
		} else {
			return file_put_contents( $file, $message );
		}
	}

}